<?php

namespace Drupal\openstory\Plugin\rest\resource;

use Drupal\rest\Plugin\ResourceBase;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\rest\ResourceResponse;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a resource to get the structure of an entity.
 *
 * @RestResource(
 *   id = "entity_structure",
 *   label = @Translation("Get Entity Structure File"),
 *   uri_paths = {
 *     "canonical" =
 *   "/entity-structure/{entity_type}/{bundle}",
 *     "https://www.drupal.org/link-relations/create" =
 *   "/entity-structure/{entity_type}/{bundle}"
 *   }
 * )
 */
class GetEntityStructureFile extends ResourceBase {

  /**
   * A current user instance.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   */
  public function __construct(
      array $configuration,
      $plugin_id,
      $plugin_definition,
      array $serializer_formats,
      LoggerInterface $logger,
      AccountProxyInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
        $configuration,
        $plugin_id,
        $plugin_definition,
        $container->getParameter('serializer.formats'),
        $container->get('logger.factory')->get('custom_rest'),
        $container->get('current_user')
    );
  }

  /**
   * Responds to GET requests.
   *
   * Returns entity.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $bundle
   *   The entity bundle.
   *
   * @return \Drupal\rest\ResourceResponse
   *   The response containing entity fields.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   */
  public function get($entity_type, $bundle) {
    if (!empty($entity_type) && !empty($bundle)) {
      $entity_structure = \Drupal::service('entity_structure');
      // Check if user has permission to create nodes, comments, etc.
      if ($this->currentUser->hasPermission($entity_structure->getUserPermissionName($entity_type, $bundle))) {
        $file = $entity_structure->checkFileExists($entity_type, $bundle);
        if (empty($file)) {
          throw new BadRequestHttpException('There is no file for this entity.');
        }
        $data = $entity_structure->getEntityFile($file);
        $response = new ResourceResponse($data, 200);
        $response->addCacheableDependency($data);
        return $response;
      }
      else {
        throw new BadRequestHttpException('Access denied. You do not have the right permission.');
      }
    }
    else {
      throw new BadRequestHttpException('Parameters entity type and bundle are mandatory.');
    }
  }

}
